from time import time
from zope.interface import implements
from zope.viewlet.interfaces import IViewlet
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import memoize
from Products.CMFPlone import PloneMessageFactory as _

class Following(BrowserView):

    implements(IViewlet)
  
    def __init__(self, context, request, view, manager):
        self.context = context
        self.request = request
        self.__parent__ = view
        self.manager = manager

    def update(self):
        pass

    def name(self):
        return self.__name__

    def title(self):
        return _("Following")

    def GetFollowingStatuses(self, user_ids=None):
        """
        Take the result from GetFriendsTimeline and group the statuses by user.
        Return a list of dictionaries of the form 
        {'user':user, 'statuses':statuses}
        """
        view = self.context.restrictedTraverse('@@collective.sylvester.dashboard')
        user_status_map = {}
        userid_user_map = {}
        for status in view.GetFriendsTimeline():
            # Skip over users we are not interested in
            if (user_ids is not None) and (status.user.id not in user_ids):
                continue
            user_status_map.setdefault(status.user.id, [])
            user_status_map[status.user.id].append(status)
            userid_user_map[status.user.id] = status.user
        
        result = []
        for userid, statuses in user_status_map.items():
            result.append(dict(user=userid_user_map[userid], statuses=statuses))

        # Sort
        def mysort(a, b):
            return cmp(a['user'].screen_name, b['user'].screen_name)
        result.sort(mysort)

        return result

    render = ViewPageTemplateFile("following.pt")    



from time import time

def _cache_key_base(fun, obj, *args, **kwargs):
    """  
    Generic cache key
    """
    s_args = str(args)

    li = kwargs.items()
    li.sort()
    s_kwargs = str(li)

    #import logging
    #logger = logging.getLogger('sylvester')
    #logger.info("cache key: args = %s, kwargs = %s" % (s_args, s_kwargs))

    return "%s%s%s%s" % \
        (fun.func_name, str(obj.context.getPhysicalPath()), s_args, s_kwargs)

def cache_key_3600(fun, obj, *args, **kwargs):
    result = _cache_key_base(fun, obj, *args, **kwargs)
    return '%s%s' % (result, time()//3600)

def cache_key_60(fun, obj, *args, **kwargs):
    result = _cache_key_base(fun, obj, *args, **kwargs)
    return '%s%s' % (result, time()//60)



from time import time
from zope.interface import implements
from zope.viewlet.interfaces import IViewlet
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import memoize
from Products.CMFPlone import PloneMessageFactory as _
from collective.sylvester.utils import cache_key_60

class Replies(BrowserView):

    implements(IViewlet)
  
    def __init__(self, context, request, view, manager):
        self.context = context
        self.request = request
        self.__parent__ = view
        self.manager = manager
        self.dashboard = self.context.restrictedTraverse('@@collective.sylvester.dashboard')

    def update(self):
        pass

    def name(self):
        return self.__name__

    def title(self):
        if self.dashboard.twitter is not None:
            return "@%s" % self.dashboard.twitter.username
        return _("Mentions")

    @memoize.ram.cache(cache_key_60)
    def _GetReplies(self, username):
        """
        Cacheable companion to GetReplies
        """
        return self.dashboard.twitter.GetReplies()

    def GetReplies(self):
        return self._GetReplies(username=self.dashboard.twitter.username)

    def keepfresh(self):
        """
        We must always be ready with the answer
        """
        self.GetReplies()

    render = ViewPageTemplateFile("replies.pt")    



from time import time
from twitter import Status
from zope.interface import implements
from zope.viewlet.interfaces import IViewlet
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from plone import memoize
from Products.CMFPlone import PloneMessageFactory as _
from collective.sylvester.utils import cache_key_60, cache_key_3600

class DirectMessages(BrowserView):

    implements(IViewlet)
  
    def __init__(self, context, request, view, manager):
        self.context = context
        self.request = request
        self.__parent__ = view
        self.manager = manager
        self.dashboard = self.context.restrictedTraverse('@@collective.sylvester.dashboard')

    def update(self):
        pass

    def name(self):
        return self.__name__

    def title(self):
        return _("Direct Messages")

    @memoize.ram.cache(cache_key_60)
    def _GetDirectMessages(self, username):
        """
        Cacheable companion to GetDirectMessages
        """
        return self.dashboard.twitter.GetDirectMessages()

    def GetDirectMessages(self):
        return self._GetDirectMessages(username=self.dashboard.twitter.username)

    # This can really be cached forever but due to RAM constraints it is 
    # sensible to limit the duration.
    @memoize.ram.cache(cache_key_3600)
    def message2status(self, message):
        """
        Convert message into a status

        xxx: this is a very limited conversion!
        """
        user = self.dashboard.twitter.GetUser(message.sender_id)
        status = Status(text=message.text, 
            created_at=message.created_at,
            user=user)
        return status

    def keepfresh(self):
        """
        We must always be ready with the answer
        """     
        self.GetDirectMessages()

    render = ViewPageTemplateFile("directmessages.pt")    



from Acquisition import aq_inner
from collective.sylvester.browser.kssviews import BaseSylvesterKssView, \
    kssactionplus

class FollowingKssView(BaseSylvesterKssView):

    @kssactionplus
    def refreshFollowing(self, user_ids=None): 
        """
        Refresh a a few friendlets.
        """
        context = aq_inner(self.context)
        ksscore = self.getCommandSet('core')

        view = context.restrictedTraverse('@@collective.sylvester.dashboard')
        mapping = view.GetUsersStatuses(user_ids=user_ids)
        for di in mapping.items():
            user = di['user']
            statuses = ['statuses']
            html = self.viewletMacroContent(                    
                    'collective.sylvester.friendletmanager', 
                    'collective.sylvester.friendlet', 
                    'main',
                    friend=user,
                    statuses=statuses)
            selector = ksscore.getHtmlIdSelector(
                'collective-sylvester-friendlet-friend%s' % user_id) 
            ksscore.replaceHTML(selector, html)
